﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;

using Telerik.Windows.Controls;

namespace FileExplorerExample {

    public class FileExplorer : RadTreeView {
        private object newItem = null;
        static FileExplorer() {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(FileExplorer),
                                                    new FrameworkPropertyMetadata(typeof(FileExplorer)));
        }

        //this method will be called after layout updated
        private void SelectNewItem(object sender, EventArgs e) {
            ExpandAndSelectItem(newItem);
            this.LayoutUpdated -= SelectNewItem;
            newItem = null;
        }
        public void SelectNewAddedItem(object itemToSelect) {
            this.LayoutUpdated += new EventHandler(SelectNewItem);
            newItem = itemToSelect;
        }
        public bool ExpandAndSelectItem(object itemToSelect) {
            for(int i = 0; i < Items.Count; i++) {
                // try to get the item Container  
                RadTreeViewItem childItemContainer = ItemContainerGenerator.ContainerFromIndex(i) as RadTreeViewItem;
                // the item container maybe null if it is still not generated from the runtime  
                if(childItemContainer != null) {
                    childItemContainer.IsSelected = false;
                    if(childItemContainer.Item == itemToSelect) {
                        childItemContainer.IsSelected = true;
                        childItemContainer.BringIntoView();
                        childItemContainer.Focus();
                        childItemContainer.UpdateLayout();
                        return true;
                    }

                    childItemContainer.IsExpanded = true;

                    this.Dispatcher.BeginInvoke(new Action(() => {
                        ExpandAndSelectItem(childItemContainer, itemToSelect);
                    }), System.Windows.Threading.DispatcherPriority.ApplicationIdle);
                }
            }

            return false;
        }

        public bool ExpandAndSelectItem(ItemsControl itemsControl, object itemToSelect) {
            for(int i = 0; i < itemsControl.Items.Count; i++) {
                // try to get the item Container  
                RadTreeViewItem childItemContainer = itemsControl.ItemContainerGenerator.ContainerFromIndex(i) as RadTreeViewItem;
                // the item container maybe null if it is still not generated from the runtime  
                if(childItemContainer != null) {
                    if(childItemContainer.IsSelected)
                        childItemContainer.IsSelected = false;
                    if(childItemContainer.Item is File) {
                        var p = ((DirectoryFile)childItemContainer.Item).FullPath;
                        var u = ((DirectoryFile)itemToSelect).FullPath;

                        if(p == u) {
                            childItemContainer.IsSelected = true;
                            childItemContainer.BringIntoView();
                            childItemContainer.Focus();
                            return true;
                        }
                    }

                    if(childItemContainer.Items.Count > 0)
                        ExpandAndSelectItem(childItemContainer, itemToSelect);
                }
            }

            return false;
        }

        //public RadTreeView FolderTree { get; set; }

        //public override void OnApplyTemplate()
        //{
        //    base.OnApplyTemplate();
   
        // "folderTree" is child of the Grid which is actually templated child of FileExplorer
        // ==> base.GetTemplateChild("folderTree") will return null.
        //    FolderTree = base.GetTemplateChild("folderTree") as RadTreeView;
        //}
    }

    public class FileDataTemplateSelector : DataTemplateSelector {

        private HierarchicalDataTemplate directoryTemplate;
        private HierarchicalDataTemplate directoryInnerTemplate;
        private DataTemplate fileTemplate;

        public FileDataTemplateSelector() {
        }

        public override DataTemplate SelectTemplate(object item, DependencyObject container) {
            if(item is DirectoryFile && ((DirectoryFile)item).IsRoot == true && 
                ((DirectoryFile)item).IsDirectory == true)
                return this.directoryTemplate;
            else if(item is DirectoryFile && ((DirectoryFile)item).IsRoot == false && 
                    ((DirectoryFile)item).IsDirectory == true)
                return this.directoryInnerTemplate;
            else if(item is DirectoryFile && ((DirectoryFile)item).IsDirectory == false)
                return this.fileTemplate;

            return null;
        }

        public HierarchicalDataTemplate DirectoryTemplate {
            get {
                return this.directoryTemplate;
            }
            set {
                this.directoryTemplate = value;
            }
        }
        public HierarchicalDataTemplate DirectoryInnerTemplate {
            get {
                return this.directoryInnerTemplate;
            }
            set {
                this.directoryInnerTemplate = value;
            }
        }
        public DataTemplate FileTemplate {
            get {
                return this.fileTemplate;
            }
            set {
                this.fileTemplate = value;
            }
        }
    }

    public class ItemStyleSelector : StyleSelector {
        public override Style SelectStyle(object item, DependencyObject container) {
            if(item is DirectoryFile && ((DirectoryFile)item).IsRoot == true && 
                ((DirectoryFile)item).IsDirectory == true)
                return this.DirectoryStyle;
            else if(item is DirectoryFile && ((DirectoryFile)item).IsRoot == false && 
                    ((DirectoryFile)item).IsDirectory == true)
                return this.DirectoryInnerStyle;
            else if(item is DirectoryFile && ((DirectoryFile)item).IsDirectory == false)
                return this.FileStyle;
            return base.SelectStyle(item, container);
        }

        public Style DirectoryStyle {
            get;
            set;
        }
        public Style DirectoryInnerStyle {
            get;
            set;
        }
        public Style FileStyle {
            get;
            set;
        }
    }

    public class DirectoryFile : INotifyPropertyChanged, IPath {
        private bool isChecked, isSelected, isExpanded;

        public DirectoryFile(string fullPath, string name, bool isRoot, bool isDirectory, IPath parent) {
            this.FullPath = fullPath;
            this.Name = name;
            this.IsRoot = isRoot;
            this.IsDirectory = isDirectory;
            this.Parent = parent;
            this.Childs = new ObservableCollection<DirectoryFile>();
        }

        public string GetFullPath() {
            if(this.Parent != null)
                return this.Parent.GetFullPath() + "|" + this.ToString();
            return this.ToString();
        }

        public override string ToString() {
            return this.Name;
        }

        public bool IsExpanded {
            get {
                return this.isExpanded;
            }
            set {
                if(this.isExpanded != value) {
                    this.isExpanded = value;
                    OnNotifyPropertyChanged("IsExpanded");
                }
            }
        }
        public bool IsChecked {
            get {
                return this.isChecked;
            }
            set {
                if(this.isChecked != value) {
                    this.isChecked = value;
                    OnNotifyPropertyChanged("IsChecked");
                }
            }
        }
        public bool IsSelected {
            get {
                return this.isSelected;
            }
            set {
                if(this.isSelected != value) {
                    this.isSelected = value;
                    OnNotifyPropertyChanged("IsSelected");
                }
            }
        }

        public IPath Parent {
            get;
            set;
        }

        public string FullPath {
            get;
            set;
        }
        public string Name {
            get;
            set;
        }
        public bool IsRoot {
            get;
            set;
        }

        public bool IsDirectory {
            get;
            set;
        }

        public ObservableCollection<DirectoryFile> Childs {
            get;
            set;
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnNotifyPropertyChanged(string propertyName) {
            if(PropertyChanged != null) {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }

    public interface IPath {
        string GetFullPath();

        IPath Parent {
            get;
            set;
        }
    }
}
